@@ -1,7 +1,7 @@
 from typing import List, Callable
 import os
-import json
 import ufal.udpipe as udpipe
+from lemmagen3 import Lemmatizer
 import serverfiles
 from nltk import stem
 from requests.exceptions import ConnectionError
@@ -14,7 +14,7 @@ from orangecontrib.text.misc import wait_nltk_data
 from orangecontrib.text.preprocess import Preprocessor, TokenizedPreprocessor
 
 __all__ = ['BaseNormalizer', 'WordNetLemmatizer', 'PorterStemmer',
-           'SnowballStemmer', 'UDPipeLemmatizer']
+           'SnowballStemmer', 'UDPipeLemmatizer', 'LemmagenLemmatizer']
 
 
 class BaseNormalizer(TokenizedPreprocessor):
@@ -24,6 +24,10 @@ class BaseNormalizer(TokenizedPreprocessor):
     """
     normalizer = NotImplemented
 
+    def __init__(self):
+        # cache already normalized string to speedup normalization
+        self._normalization_cache = {}
+
     def __call__(self, corpus: Corpus, callback: Callable = None) -> Corpus:
         if callback is None:
             callback = dummy_callback
@@ -33,7 +37,16 @@ class BaseNormalizer(TokenizedPreprocessor):
 
     def _preprocess(self, string: str) -> str:
         """ Normalizes token to canonical form. """
-        return self.normalizer(string)
+        if string in self._normalization_cache:
+            return self._normalization_cache[string]
+        self._normalization_cache[string] = norm_string = self.normalizer(string)
+        return norm_string
+
+    def __getstate__(self):
+        d = self.__dict__.copy()
+        # since cache can be quite big, empty cache before pickling
+        d["_normalization_cache"] = {}
+        return d
 
 
 class WordNetLemmatizer(BaseNormalizer):
@@ -52,13 +65,12 @@ class PorterStemmer(BaseNormalizer):
 
 class SnowballStemmer(BaseNormalizer):
     name = 'Snowball Stemmer'
-    supported_languages = [l.capitalize() for l in stem.SnowballStemmer.languages]
+    supported_languages = [l.capitalize() for l in
+                           stem.SnowballStemmer.languages]
 
     def __init__(self, language='English'):
-        self.normalizer = stem.SnowballStemmer(language.lower())
-
-    def _preprocess(self, token):
-        return self.normalizer.stem(token)
+        super().__init__()
+        self.normalizer = stem.SnowballStemmer(language.lower()).stem
 
 
 def language_to_name(language):
@@ -70,7 +82,7 @@ def file_to_name(file):
 
 
 def file_to_language(file):
-    return file[:file.find('ud')-1]\
+    return file[:file.find('ud') - 1] \
         .replace('-', ' ').replace('_', ' ').capitalize()
 
 
@@ -119,6 +131,7 @@ class UDPipeLemmatizer(BaseNormalizer):
     name = 'UDPipe Lemmatizer'
 
     def __init__(self, language='English', use_tokenizer=False):
+        super().__init__()
         self.__language = language
         self.__use_tokenizer = use_tokenizer
         self.models = UDPipeModels()
@@ -154,8 +167,7 @@ class UDPipeLemmatizer(BaseNormalizer):
         sentence = udpipe.Sentence()
         sentence.addWord(token)
         self.__model.tag(sentence, self.__model.DEFAULT)
-        output = self.__output_format.writeSentence(sentence)
-        return json.loads(output)['nodes'][0]['properties']['lemma']
+        return sentence.words[1].lemma
 
     def __normalize_document(self, document: str) -> List[str]:
         tokens = []
@@ -165,8 +177,64 @@ class UDPipeLemmatizer(BaseNormalizer):
         sentence = udpipe.Sentence()
         while tokenizer.nextSentence(sentence, error):
             self.__model.tag(sentence, self.__model.DEFAULT)
-            output = self.__output_format.writeSentence(sentence)
+            # 1: is used because words[0] is the root required by the dependency trees
+            tokens.extend([w.lemma for w in sentence.words[1:]])
             sentence = udpipe.Sentence()
-            tokens.extend([t['properties']['lemma']
-                           for t in json.loads(output)['nodes']])
         return tokens
+
+    def __getstate__(self):
+        """
+        This function remove udpipe.Model that cannot be pickled
+
+        Note: __setstate__ is not required since we do not make any harm if
+              model is not restored. It will be loaded on __call__
+        """
+        state = super().__getstate__()
+        # Remove the unpicklable Model and output format.
+        state['_UDPipeLemmatizer__model'] = None
+        state['_UDPipeLemmatizer__output_format'] = None
+        return state
+
+
+class LemmagenLemmatizer(BaseNormalizer):
+    name = 'Lemmagen Lemmatizer'
+    lemmagen_languages = {
+        "Bulgarian": "bg",
+        "Croatian": "hr",
+        "Czech": "cs",
+        "English": "en",
+        "Estonian": "et",
+        "Farsi/Persian": "fa",
+        "French": "fr",
+        "German": "de",
+        "Hungarian": "hu",
+        "Italian": "it",
+        "Macedonian": "mk",
+        "Polish": "pl",
+        "Romanian": "ro",
+        "Russian": "ru",
+        "Serbian": "sr",
+        "Slovak": "sk",
+        "Slovenian": "sl",
+        "Spanish": "es",
+        "Ukrainian": "uk"
+    }
+
+    def __init__(self, language='English'):
+        super().__init__()
+        self.language = language
+        self.lemmatizer = None
+
+    def __call__(self, corpus: Corpus, callback: Callable = None) -> Corpus:
+        # lemmagen3 lemmatizer is not picklable, define it on call and discard it afterward
+        self.lemmatizer = Lemmatizer(self.lemmagen_languages[self.language])
+        output_corpus = super().__call__(corpus, callback)
+        self.lemmatizer = None
+        return output_corpus
+
+    def normalizer(self, token):
+        assert self.lemmatizer is not None
+        t = self.lemmatizer.lemmatize(token)
+        # sometimes Lemmagen returns an empty string, return original tokens
+        # in this case
+        return t if t else token
