@@ -1,10 +1,16 @@
 import functools
+from typing import Any
+
+import numpy as np
 
 from AnyQt import QtGui, QtCore
 from AnyQt.QtCore import pyqtSignal, QSize
 from AnyQt.QtWidgets import (QVBoxLayout, QButtonGroup, QRadioButton,
                              QGroupBox, QTreeWidgetItem, QTreeWidget,
                              QStyleOptionViewItem, QStyledItemDelegate, QStyle)
+from Orange.widgets.utils.concurrent import TaskState, ConcurrentWidgetMixin
+
+from gensim.models import CoherenceModel
 
 from Orange.widgets import settings
 from Orange.widgets import gui
@@ -13,7 +19,7 @@ from Orange.widgets.widget import OWWidget, Input, Output, Msg
 from Orange.data import Table
 from orangecontrib.text.corpus import Corpus
 from orangecontrib.text.topics import Topic, LdaWrapper, HdpWrapper, LsiWrapper
-from orangecontrib.text.widgets.utils.concurrent import asynchronous
+from orangecontrib.text.topics.topics import GensimWrapper
 
 
 class TopicWidget(gui.OWComponent, QGroupBox):
@@ -96,7 +102,16 @@ def require(attribute):
     return decorator
 
 
-class OWTopicModeling(OWWidget):
+def _run(corpus: Corpus, model: GensimWrapper, state: TaskState):
+    def callback(i: float):
+        state.set_progress_value(i * 100)
+        if state.is_interruption_requested():
+            raise Exception
+
+    return model.fit_transform(corpus.copy(), chunk_number=100, on_progress=callback)
+
+
+class OWTopicModeling(OWWidget, ConcurrentWidgetMixin):
     name = "Topic Modelling"
     description = "Uncover the hidden thematic structure in a corpus."
     icon = "icons/TopicModeling.svg"
@@ -110,7 +125,7 @@ class OWTopicModeling(OWWidget):
         corpus = Input("Corpus", Corpus)
 
     class Outputs:
-        corpus = Output("Corpus", Table)
+        corpus = Output("Corpus", Table, default=True)
         selected_topic = Output("Selected Topic", Topic)
         all_topics = Output("All Topics", Table)
 
@@ -137,11 +152,18 @@ class OWTopicModeling(OWWidget):
     class Warning(OWWidget.Warning):
         less_topics_found = Msg('Less topics found than requested.')
 
+    class Error(OWWidget.Error):
+        unexpected_error = Msg("{}")
+
     def __init__(self):
         super().__init__()
+        ConcurrentWidgetMixin.__init__(self)
+
         self.corpus = None
         self.learning_thread = None
         self.__pending_selection = self.selection
+        self.perplexity = "n/a"
+        self.coherence = "n/a"
 
         # Commit button
         gui.auto_commit(self.buttonsArea, self, 'autocommit', 'Commit', box=False)
@@ -168,6 +190,11 @@ class OWTopicModeling(OWWidget):
         self.toggle_widgets()
         method_layout.addStretch()
 
+        box = gui.vBox(self.controlArea, "Topic evaluation")
+        gui.label(box, self, "Log perplexity: %(perplexity)s")
+        gui.label(box, self, "Topic coherence: %(coherence)s")
+        self.controlArea.layout().insertWidget(1, box)
+
         # Topics description
         self.topic_desc = TopicViewer()
         self.topic_desc.topicSelected.connect(self.send_topic_by_id)
@@ -199,43 +226,43 @@ class OWTopicModeling(OWWidget):
             widget.setVisible(i == self.method_index)
 
     def apply(self):
-        self.learning_task.stop()
-        if self.corpus is not None:
-            self.learning_task()
-        else:
-            self.on_result(None)
-
-    @asynchronous
-    def learning_task(self):
-        return self.model.fit_transform(self.corpus.copy(), chunk_number=100,
-                                        on_progress=self.on_progress)
-
-    @learning_task.on_start
-    def on_start(self):
-        self.Warning.less_topics_found.clear()
-        self.progressBarInit()
         self.topic_desc.clear()
-
-    @learning_task.on_result
-    def on_result(self, corpus):
-        self.progressBarFinished()
-        self.Outputs.corpus.send(corpus)
-        if corpus is None:
+        if self.corpus is not None:
+            self.Warning.less_topics_found.clear()
+            self.start(_run, self.corpus, self.model)
+        else:
             self.topic_desc.clear()
+            self.Outputs.corpus.send(None)
             self.Outputs.selected_topic.send(None)
             self.Outputs.all_topics.send(None)
-        else:
-            self.topic_desc.show_model(self.model)
-            if self.__pending_selection:
-                self.topic_desc.select(self.__pending_selection)
-                self.__pending_selection = None
-            if self.model.actual_topics != self.model.num_topics:
-                self.Warning.less_topics_found()
-            self.Outputs.all_topics.send(self.model.get_all_topics_table())
 
-    @learning_task.callback
-    def on_progress(self, p):
-        self.progressBarSet(100 * p)
+    def on_done(self, corpus):
+        self.Outputs.corpus.send(corpus)
+        pos_tags = self.corpus.pos_tags is not None
+        self.topic_desc.show_model(self.model, pos_tags=pos_tags)
+        if self.__pending_selection:
+            self.topic_desc.select(self.__pending_selection)
+            self.__pending_selection = None
+
+        if self.model.actual_topics != self.model.num_topics:
+            self.Warning.less_topics_found()
+
+        if self.model.name == "Latent Dirichlet Allocation":
+            bound = self.model.model.log_perplexity(corpus.ngrams_corpus)
+            self.perplexity = "{:.5f}".format(np.exp2(-bound))
+        cm = CoherenceModel(
+            model=self.model.model, texts=corpus.tokens, corpus=corpus, coherence="c_v"
+        )
+        coherence = cm.get_coherence()
+        self.coherence = "{:.5f}".format(coherence)
+
+        self.Outputs.all_topics.send(self.model.get_all_topics_table())
+
+    def on_exception(self, ex: Exception):
+        self.Error.unexpected_error(str(ex))
+
+    def on_partial_result(self, result: Any) -> None:
+        pass
 
     def send_report(self):
         self.report_items(*self.widgets[self.method_index].report_model())
@@ -251,15 +278,17 @@ class OWTopicModeling(OWWidget):
 
 class TopicViewerTreeWidgetItem(QTreeWidgetItem):
     def __init__(self, topic_id, words, weights, parent,
-                 color_by_weights=False):
+                 color_by_weights=False, pos_tags=False):
         super().__init__(parent)
         self.topic_id = topic_id
         self.words = words
         self.weights = weights
         self.color_by_weights = color_by_weights
+        self.pos_tags = pos_tags
 
         self.setText(0, '{:d}'.format(topic_id + 1))
-        self.setText(1, ', '.join(self._color(word, weight)
+        self.setText(1, ', '.join(self._color(word.rsplit("_", 1)[0], weight) if
+                                  self.pos_tags else self._color(word, weight)
                                   for word, weight in zip(words, weights)))
 
     def _color(self, word, weight):
@@ -296,7 +325,7 @@ class TopicViewer(QTreeWidget):
         for i in range(self.columnCount()):
             self.resizeColumnToContents(i)
 
-    def show_model(self, topic_model):
+    def show_model(self, topic_model, pos_tags=False):
         self.clear()
         if topic_model.model:
             for i in range(topic_model.num_topics):
@@ -304,7 +333,8 @@ class TopicViewer(QTreeWidget):
                 if words:
                     it = TopicViewerTreeWidgetItem(
                         i, words, weights, self,
-                        color_by_weights=topic_model.has_negative_weights)
+                        color_by_weights=topic_model.has_negative_weights,
+                        pos_tags=pos_tags)
                     self.addTopLevelItem(it)
 
             self.resize_columns()
@@ -381,4 +411,3 @@ if __name__ == '__main__':
     widget.set_data(Corpus.from_file('deerwester'))
     widget.show()
     app.exec()
-    widget.saveSettings()
